#!/bin/sh
#
# make_xprint_binary_tarball.sh - packager script for Xprint stuff
#
# $Xorg: make_xprint_binary_tarball.sh,v 1.1 2002/07/07 07:41:52 gisburn Exp $
#
# This script creates a binary tarball for Xprint stuff (Xprt, Xprt config
# files and Xprint helper applications.
#
# Written by Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
#

# These should be provided by Imakefile
#XTOP=..
#XPRINTDIR=/usr/openwin/lib/X11/xserver
#XBINDIR=/usr/openwin/bin
#
# XPDESTTARFILE may be optionally set as a command line argument to give the 
# destination filepath for the tarball, otherwise it will be found in /tmp.
# Note that the tarball is automatically compressed (to .gz, if gzip is
# available, otherwise to .Z); your name for XPDESTTARFILE should reflect this.

# Check if mandatory env vars have been set
if [ "${XTOP}"      = "" ] ; then echo "XTOP env var not set"      ; exit 1 ; fi
if [ "${XPRINTDIR}" = "" ] ; then echo "XPRINTDIR env var not set" ; exit 1 ; fi
if [ "${XBINDIR}"   = "" ] ; then echo "XBINDIR env var not set"   ; exit 1 ; fi

# Do not edit
# REL_* vars are paths relative to '/'
# Thanks to the "wonders" of some /bin/sh I cannot use ...
# REL_XPRINTDIR=${XPRINTDIR#*/}
# REL_XBINDIR=${XBINDIR#*/}
# ... ;-(
REL_XPRINTDIR="`echo ${XPRINTDIR} | sed 's/\///'`"
REL_XBINDIR="`echo ${XBINDIR} | sed 's/\///'`"

XPINSTALL_SESSION="`date +%y%m%d%H%M%S`"
XPINSTALLDIR=/tmp/xpinstall_${XPINSTALL_SESSION}
XPLOGFILE=/tmp/xpinstall_${XPINSTALL_SESSION}.log
XPTARFILE=/tmp/xprint_server_${XPINSTALL_SESSION}.tar

DO_BUILD=false

# List of files/dirs which should be put into the tarball
BINFILELIST=" \
  ${REL_XBINDIR}/Xprt \
  ${REL_XBINDIR}/xplsprinters \
  ${REL_XBINDIR}/xphelloworld \
  ${REL_XBINDIR}/xpawhelloworld \
  ${REL_XBINDIR}/xpsimplehelloworld \
  etc \
  ${REL_XPRINTDIR}"

step=0

inc_step_counter()
{
  step=`expr ${step} + 1`
}

has_gzip()
{
  for i in `echo $PATH | tr ":" " "` 
  do
    if [ -x "${i}/gzip" ] ; then
      return 0
    fi
  done

  return 1
}

# Step 1:
# Build xc/ tree...
if ${DO_BUILD} ; then
  inc_step_counter
  echo "## Step ${step}: Building..."
  (
    (
      cd ${XTOP}
      if make World 2>&1 | tee -a buildlog.log >&${XPLOGFILE} ; then
        echo "Build OK."
      else
        echo "Build failed:"
        echo "-- snip --"
        tail ${XPLOGFILE}
        echo "-- snip --"
        exit 1
      fi
    )
  )
  echo "##         Build completed DONE."
fi # ${DO_BUILD}

# Step 2:
# Verification...
inc_step_counter
echo "## Step ${step}: Verifying..."
(
  status=true

  if [ ! -x ${XTOP}/programs/Xserver/Xprt ] ; then
    echo "error: X print server (${XTOP}/programs/Xserver/Xprt) binary missing."
    status=false
  fi

  if [ ! -x ${XTOP}/programs/xplsprinters/xplsprinters ] ; then
    echo "error: ${XTOP}/programs/xplsprinters/xplsprinters binary missing."
    status=false
  fi

  if [ ! -x ${XTOP}/programs/xphelloworld/xpawhelloworld/xpawhelloworld ] ; then
    echo "error: ${XTOP}/programs/xphelloworld/xpawhelloworld/xpawhelloworld binary missing."
    status=false
  fi

  if [ ! -x ${XTOP}/programs/xphelloworld/xphelloworld/xphelloworld ] ; then
    echo "error: ${XTOP}/programs/xphelloworld/xphelloworld/xphelloworld binary missing."
    status=false
  fi

  if [ ! -x ${XTOP}/programs/xphelloworld/xpsimplehelloworld/xpsimplehelloworld  ] ; then
    echo "error: ${XTOP}/programs/xphelloworld/xpsimplehelloworld/xpsimplehelloworld binary missing."
    status=false
  fi

  if ${status} ; then
    echo "No errors found."
  else
    echo "Verification failed."
    exit 1
  fi
)
[ $? != 0 ] && exit 1;
echo "##         Verification DONE."

# Step 3:
# Install into temp. location
inc_step_counter
echo "## Step ${step}: Install into temporary location..."
(
  mkdir ${XPINSTALLDIR}
  if [ ! -d ${XPINSTALLDIR} ] ; then
    echo "Could not create temp. installation dir ${XPINSTALLDIR}."
    exit 1
  fi
  
  (
    cd ${XTOP}
    if make install DESTDIR=${XPINSTALLDIR} >>${XPLOGFILE} 2>&1 ; then
      echo "Installation OK."
    else
      echo "Installation failed:"
      echo "-- snip --"
      tail ${XPLOGFILE}
      echo "-- snip --"
      exit 1
    fi
  )
)
[ $? != 0 ] && exit 1;
echo "##         Temp. installation DONE."

# Step 4:
# Verify installation
inc_step_counter
echo "## Step ${step}: Verifying installation..."
(
  cd ${XPINSTALLDIR}
  status=true
  for i in ${BINFILELIST} ; do
    if [ ! -x ${i} ] ; then
      echo "File ${i} missing."
      status=false
    fi
  done
  
  if ${status} ; then
    echo "No errors found."
  else
    echo "Verification failed."
    exit 1
  fi
)
[ $? != 0 ] && exit 1;
echo "##        Verification of installation DONE."

# Step 5.a:
# Create binary tarball
inc_step_counter
echo "## Step ${step}: Create binary tarball..."
(
  cd ${XPINSTALLDIR}
  if has_gzip ; then
    tar -cf - ${BINFILELIST} | gzip --best --stdout >${XPTARFILE}.gz
  else
    tar -cf - ${BINFILELIST} | compress >${XPTARFILE}.Z
  fi
)
[ $? != 0 ] && exit 1;
echo "##        Binary tarball creation DONE."

# Step 5.b:
# Copy binary tarball to destination, if required.
if [ "${XPDESTTARFILE}" != "" ] ; then
    inc_step_counter
    echo "## Step ${step}: Copying binary tarball to destination..."
    (
    if has_gzip ; then
	mv ${XPTARFILE}.gz ${XPDESTTARFILE}
    else
	mv ${XPTARFILE}.Z ${XPDESTTARFILE}
    fi
    )
    echo "##        Copying tarball to destination DONE."
fi

# Step 6:
# Cleaning up
inc_step_counter
echo "## Step ${step}: Cleaning up..."
(
  # Be carefull... =:-)
  if [ "${XPINSTALLDIR}" != "/" ] ; then
    rm -Rf ${XPINSTALLDIR}
  fi
)
echo "##        Cleanup done."


# Done.
echo "#### Binary tarball is now available at "
if [ "${XPDESTTARFILE}" != "" ] ; then
    ls -l ${XPDESTTARFILE}*
else
    ls -l ${XPTARFILE}*
fi
echo "#### All done."

exit 0
# EOF.
